#include	<stdio.h>
#include	<string.h>

#include	"types.h"
#include	"getarg.h"
/*
    getarg --- Parse Argument line

    Parameters: argc	INT	Argument count, elements of argv[], 
				as given by main()

		argv[]	CHAR**	Array of strings containing the broken dowwm
				command line argv[0] contains the program name.

		allopts	CHAR*	String containing all option CHARacters
				without leading '-'. Options that may comprise
				a value must be followed by a colon.
				Example: "abc:de:f"
                                Options not listed in allopts are treated as
				value-less options.

		opt		Option to scan for (CHARacter) or a negative
				INTeger value starting with -1 to scan for 
				an argument.

		defval	CHAR*	value to be returned if an option is specified
				without a value. Meaningless with parameters

		notfound CHAR*	value to be returned if the option/parameter
				is not found during the scan. Returned also
				if a requested parameter is not present.
*/

CHAR *getarg (INT argc, CHAR *argv[], CHAR *allopts, INT opt, 
              CHAR *defval, CHAR *notfound)
{
    INT i;
    CHAR *ptr, *otv;

    if (opt >= 0) {
        if ((otv=strchr(allopts,opt)) == NULL) 
            otv = "";
        else
            otv++;

        for (i = 1; i < argc; i++) {
            ptr = argv[i];
            if (*ptr++ == '-') {
                if (*ptr == '\0') continue;	/* dash-only ignored */
                if (*ptr++ == (CHAR)opt) {
                    if (*otv != ':') return (defval);
                    if (*ptr == '\0') {
                        if (i == (argc-1)) {

                            return (defval);
                        }
                        else {
                            ptr = argv[i+1];
                            if (*ptr == '-') 
                                return (defval);
                            else
                                return (ptr);
                        }
                    }
                    else {
                        return (ptr);
                    }
                }
            }
        }
        return (notfound);
    }
    else {
        INT last;

        for (i = 1, last = 0; i < argc; i++) {
            ptr = argv[i];
            if (*ptr++ != '-') break;	/* option list stops with first argument */
            if (*ptr == '\0') break;	/* dash-only treated as an argument */
            last = i;
            if ((otv=strchr(allopts,*ptr++)) != NULL)
                if (*(++otv) == ':')
                    if (*ptr == '\0')
                        if (i < (argc-1))
                            if (*argv[i+1] != '-') last++;
        }
        if (last-opt >= argc) return (notfound);
        return (argv[last-opt]);
    }
}

#ifdef TEST
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
	Multi-thread "strtok" (identical behaviour) but no static
	storage. All context necessary to deal with athread is stored in
	the "context" pointer
*/
CHAR *xstrtok (CHAR *buf, CHAR *pat, CHAR **context)
{
    CHAR *ptr, *result;

    if (context  == NULL) return (NULL);
    if (*pat == '\0') return (*context=NULL);
    if (buf != NULL) *context = buf;
    if ((result = *context) == NULL) return (NULL);

    for (; *result != '\0'; result++)
        if (strchr(pat,(INT)*result) == NULL) break;
    if (*result == '\0') return (*context=NULL);

    ptr = strpbrk(result, pat);

    if (ptr != NULL) *ptr++ = '\0';
    *context = ptr;

    return(result);        
}
/*
	Basically the same a "xstrtok", but doesn't modify the contents of
	the input buffer. Instead each aquired toke is copied to dynamically
	allocated buffers (by using "malloc"). The caller is responsible
	for freeing these buffers if needed.
*/
CHAR *zstrtok (CHAR *buf, CHAR *pat, CHAR **context)
{
    CHAR *ptr, *temp, *result;
    INT len;

    if (context  == NULL) return (NULL);
    if (*pat == '\0') return (*context=NULL);
    if (buf != NULL) *context = buf;
    if ((temp = *context) == NULL) return (NULL);

    for (; *temp != '\0'; temp++)
        if (strchr(pat,(INT)*temp) == NULL) break;
    if (*temp == '\0') return (*context=NULL);

    ptr = strpbrk(temp, pat);

    if (ptr == NULL) {
        len = strlen(temp);
        *context = NULL;
    }
    else {
        len = (INT)(ptr - temp);
        *context = (ptr+1);
    }

    result = (CHAR *)malloc(len+1);

    if (result == NULL)
        *context = NULL;
    else {
        memcpy(result,temp,len);
        *(result+len) = '\0';
    }

    return(result);        
}

#ifdef DEBUG

INT main (INT argc, CHAR *argv[])
{
    CHAR *p1, *p2, *p3, *ctx2, *ctx3, *a1, *a2, *a3, *b1, *b2, *b3;

    if (argc < 3) exit (0);

    a1 = b1 = (CHAR *)malloc(strlen(argv[1])+1);
    a2 = b2 = (CHAR *)malloc(strlen(argv[1])+1);
    a3 = b3 = (CHAR *)malloc(strlen(argv[1])+1);
    strcpy(a1, argv[1]);
    strcpy(a2, argv[1]);
    strcpy(a3, argv[1]);

    do {
        p1 =  strtok (a1,argv[2]);
        p2 = xstrtok (a2,argv[2],&ctx2);
        p3 = zstrtok (a3,argv[2],&ctx3);
        printf(" strtok: [%s] [%s]\n"
               "xstrtok: [%s] [%s]\n"
               "zstrtok: [%s] [%s]\n",p1,b1,p2,b2,p3,b3);
        a1 = a2 = a3 = NULL;
    } while (p1 != NULL || p2 != NULL || p3 != NULL);

    return (0);
}
#endif
static CHAR **slice (CHAR *alist)
{
    CHAR *ptr, *ctx;
    CHAR **res;
    INT  numargs = 0;

    for (ptr  = zstrtok(alist,"\t ",&ctx); 
         ptr != NULL;
         ptr  = zstrtok(NULL,"\t ",&ctx)) {
        free(ptr);
        numargs++;
    }

    if ((res = (CHAR **)malloc(sizeof(CHAR **)*(numargs+1))) == NULL) 
        return (NULL);

    if ((res[0] = (CHAR *)malloc(32)) == NULL) 
        return (NULL);

    sprintf(res[0],"%d",numargs+1);

    numargs = 1;

    for (ptr  = xstrtok(alist,"\t ",&ctx);
         ptr != NULL;
         ptr = xstrtok(NULL,"\t ",&ctx)) {
        res[numargs++] = ptr;
    }
    return(res);
}


INT main (INT argc, CHAR *argv[])
{
    CHAR **av, *res;
    INT na, i;

    if (argc < 6) {
        puts("Insufficient number of arguments (< 5)");
        exit (1);
    }

    av = slice(argv[1]);

    printf("%d arguments in test string ",atoi(av[0])-1);
    for (i=1; i<atoi(av[0]); i++) printf("[%s] ",av[i]);
    printf("\nOption definition string is [%s]\n",argv[2]);
    if (sscanf(argv[3],"%d",&na) == 1 && na < 0) {
        printf("Looking for argument [%d], default: [%s]\n",-na,argv[5]);
        res = getarg(atoi(av[0]), av, argv[2], na      , argv[4], argv[5]);
    }
    else {
        printf("Looking for option [%c], defaults: [%s] [%s]\n",
               *argv[3],argv[4],argv[5]);
        res = getarg(atoi(av[0]), av, argv[2], *argv[3], argv[4], argv[5]);
    }
    if (strlen(res)) 
        puts(res);

    return(0);
}
#endif
